package de.advolux.de;

import org.junit.*;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.io.File;
import java.io.IOException;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class DatabaseHelperTest {

    static DatabaseHelper databaseHelper = null;

    @BeforeClass
    public static void Begin() {
        String db_path = System.getProperty("user.home") + "/dbtest";
        File path = new File(db_path);
        path.mkdirs();
        File db = new File(db_path + "/test.objectdb");
        db.delete();
        databaseHelper = new DatabaseHelper(db.getAbsolutePath());
    }

    @AfterClass
    public static void End() throws IOException {
        databaseHelper.close();
    }

    @Test
    public void TestQuery() {
        System.out.println("db version: " + databaseHelper.GetVersion());
        EntityManager manager = databaseHelper.GetManager();

        TestEmbeddedClass embedd1 = new TestEmbeddedClass(1,2);
        TestEmbeddedClass embedd2 = new TestEmbeddedClass(2,3);
        TestEmbeddedClass embedd3 = new TestEmbeddedClass(3,4);

        CreateObject(manager, "blue",embedd1);
        CreateObject(manager, "green",embedd1);
        CreateObject(manager, "grey",embedd2);
        CreateObject(manager, "black",embedd3);
        CreateObject(manager, "blue",embedd3);

        List<TestEntityClass> result_embedded_set = GetQueryColor(manager, embedd1);
        List<TestEntityClass> result_color_embedded_set = GetQueryGetColorEmbedded(manager, "blue",embedd1);
        List<TestEntityClass> result_embedded_color_set = GetQueryGetEmbeddedColor(manager, "blue",embedd1);

        // the behavior in complex situations is different between 2.7.4 and 2.8.1, but I couln't strip down
        assertEquals("only embedded", 2, result_embedded_set.size());
        assertEquals("color and embedded", 1, result_color_embedded_set.size());
        assertEquals("embedded and color", 1, result_embedded_color_set.size()); // fails due to change predicate order

        databaseHelper.CloseManager(manager);
    }

    private List<TestEntityClass> GetQueryColor(EntityManager manager, TestEmbeddedClass embeddedClass) {
        CriteriaBuilder cb = manager.getCriteriaBuilder();
        CriteriaQuery<TestEntityClass> a_query = cb.createQuery(TestEntityClass.class);
        Root<TestEntityClass> a_root = a_query.from(TestEntityClass.class);
        a_query.select(a_root);

        Predicate embeddet_predicate  = cb.equal(a_root.<TestEmbeddedClass>get("embedded"), embeddedClass);
        a_query.where(embeddet_predicate);

        TypedQuery<TestEntityClass> query = manager.createQuery(a_query);
        return query.getResultList();
    }

    private List<TestEntityClass> GetQueryGetColorEmbedded(EntityManager manager, String color, TestEmbeddedClass embeddedClass) {
        CriteriaBuilder cb = manager.getCriteriaBuilder();
        CriteriaQuery<TestEntityClass> a_query = cb.createQuery(TestEntityClass.class);
        Root<TestEntityClass> a_root = a_query.from(TestEntityClass.class);
        a_query.select(a_root);

        Predicate color_predicate = cb.equal(a_root.<String>get("color"), color);
        Predicate embeddet_predicate  = cb.equal(a_root.<TestEmbeddedClass>get("embedded"), embeddedClass);
        a_query.where(color_predicate, embeddet_predicate);

        TypedQuery<TestEntityClass> query = manager.createQuery(a_query);
        return query.getResultList();
    }


    private List<TestEntityClass> GetQueryGetEmbeddedColor(EntityManager manager, String color, TestEmbeddedClass embeddedClass) {
        CriteriaBuilder cb = manager.getCriteriaBuilder();
        CriteriaQuery<TestEntityClass> a_query = cb.createQuery(TestEntityClass.class);
        Root<TestEntityClass> a_root = a_query.from(TestEntityClass.class);
        a_query.select(a_root);

        Predicate color_predicate = cb.equal(a_root.<String>get("color"), color);
        Predicate embeddet_predicate  = cb.equal(a_root.<TestEmbeddedClass>get("embedded"), embeddedClass);
        a_query.where( embeddet_predicate, color_predicate);

        TypedQuery<TestEntityClass> query = manager.createQuery(a_query);
        return query.getResultList();
    }

    private void CreateObject(EntityManager manager, String color, TestEmbeddedClass embeddedClass) {
        TestEntityClass object = new TestEntityClass();
        object.SetColor(color);
        object.SetMember(embeddedClass);
        manager.getTransaction().begin();
        manager.persist(object);
        manager.getTransaction().commit();
    }
}